@@ -188,7 +188,7 @@ class Cache:
 
     def full(self) -> bool:
         """Return whether the cache is full or not."""
-        if self.maxsize == 0:
+        if self.maxsize is None or self.maxsize <= 0:
             return False
         return len(self) >= self.maxsize
 
@@ -229,7 +229,7 @@ class Cache:
 
         return value
 
-    def get_many(self, iteratee: T_FILTER, default: t.Any = None) -> dict:
+    def get_many(self, iteratee: T_FILTER) -> dict:
         """
         Return many cache values as a ``dict`` of key/value pairs filtered by an `iteratee`.
 
@@ -242,10 +242,19 @@ class Cache:
 
         Args:
             iteratee: Iteratee to filter by.
-            default: Value to return if key doesn't exist. Defaults to ``None``.
         """
         with self._lock:
-            return {key: self.get(key, default=default) for key in self._filter(iteratee)}
+            return self._get_many(iteratee)
+
+    def _get_many(self, iteratee: T_FILTER) -> dict:
+        result = {}
+        # Convert to list since some keys may get deleted due to expiring.
+        keys = list(self._filter_keys(iteratee))
+        for key in keys:
+            value = self.get(key, default=UNSET)
+            if value is not UNSET:
+                result[key] = value
+        return result
 
     def add(self, key: t.Hashable, value: t.Any, ttl: t.Optional[T_TTL] = None) -> None:
         """
@@ -313,8 +322,12 @@ class Cache:
             items: Mapping of cache key/values to set.
             ttl: TTL value. Defaults to ``None`` which uses :attr:`ttl`.
         """
+        with self._lock:
+            self._set_many(items, ttl=ttl)
+
+    def _set_many(self, items: t.Mapping, ttl: t.Optional[T_TTL] = None) -> None:
         for key, value in items.items():
-            self.set(key, value, ttl=ttl)
+            self._set(key, value, ttl=ttl)
 
     def delete(self, key: t.Hashable) -> int:
         """
@@ -363,12 +376,16 @@ class Cache:
         Returns:
             int: Number of cache keys deleted.
         """
+        with self._lock:
+            return self._delete_many(iteratee)
+
+    def _delete_many(self, iteratee: T_FILTER) -> int:
         count = 0
         with self._lock:
             # Convert to list since we're going to be deleting keys as we iterate.
-            keys = list(self._filter(iteratee))
+            keys = list(self._filter_keys(iteratee))
             for key in keys:
-                count += self.delete(key)
+                count += self._delete(key)
         return count
 
     def delete_expired(self) -> int:
@@ -476,7 +493,7 @@ class Cache:
 
         return key, value
 
-    def _filter(self, iteratee: T_FILTER) -> t.Iterator[t.Hashable]:
+    def _filter_keys(self, iteratee: T_FILTER) -> t.Tuple[t.Hashable, ...]:
         # By default, we'll filter against cache storage.
         target: t.Iterable = self._cache
 
@@ -497,7 +514,7 @@ class Cache:
             def filter_by(key):  # type: ignore
                 return key in self._cache
 
-        yield from filter(filter_by, target)
+        return tuple(key for key in target if filter_by(key))
 
     def memoize(self, *, ttl: t.Optional[T_TTL] = None, typed: bool = False) -> T_DECORATOR:
         """
@@ -507,7 +524,9 @@ class Cache:
         Each return value from the function will be cached using the function arguments as the cache
         key. The cache object can be accessed at ``<function>.cache``. The uncached version (i.e.
         the original function) can be accessed at ``<function>.uncached``. Each return value from
-        the function will be cached using the function arguments as the cache key.
+        the function will be cached using the function arguments as the cache key. Calculate the
+        cache key for the provided function arguments for use in other operations of the
+        ``<function>.cache`` object using the function accessible at ``<function>.cache_key``
 
         Keyword Args:
             ttl (int, optional): TTL value. Defaults to ``None`` which uses :attr:`ttl`.
@@ -521,11 +540,14 @@ class Cache:
             prefix = f"{func.__module__}.{func.__name__}:"
             argspec = inspect.getfullargspec(func)
 
+            def cache_key(*args, **kwargs):
+                return _make_memoize_key(func, args, kwargs, marker, typed, argspec, prefix)
+
             if asyncio.iscoroutinefunction(func):
 
                 @wraps(func)
                 async def decorated(*args, **kwargs):
-                    key = _make_memoize_key(func, args, kwargs, marker, typed, argspec, prefix)
+                    key = cache_key(*args, **kwargs)
                     value = self.get(key, default=marker)
 
                     if value is marker:
@@ -538,7 +560,7 @@ class Cache:
 
                 @wraps(func)
                 def decorated(*args, **kwargs):
-                    key = _make_memoize_key(func, args, kwargs, marker, typed, argspec, prefix)
+                    key = cache_key(*args, **kwargs)
                     value = self.get(key, default=marker)
 
                     if value is marker:
@@ -548,6 +570,7 @@ class Cache:
                     return value
 
             decorated.cache = self
+            decorated.cache_key = cache_key
             decorated.uncached = func
 
             return decorated
